# -*- coding: cp1252 -*-
overrideIcon= "V05__VideoEnhanceAI.png"

######################################################################
#
# Name:  Create AI scaled HQ version of input videos
# Author:  Royal Render, Holger Schoenberger, Binary Alchemy
# Version: 
# Description: 
#   This scripte creates an Topaz Video Enhancer AI job for each input video.
#
# Copyright (c) Holger Schoenberger - Binary Alchemy
######################################################################



#------------------------------- Topaz commandline options ---------------------------------------------------

#use python tuples ()
modePairs = [   \
("auto"  ,   0, "Auto    - based on source height"),   \
("ahq-11", 395, "Artemis - Source is high quali"),   \
("aaa-9" ,   0, "Artemis - Source is aliased/ with moire"),   \
("amq-12", 230, "Artemis - Source is med. quali"),  \
("amqs-1",   0, "Artemis - Source is med. quali with haloing"),   \
("alq-12",   1, "Artemis - Source is low quali"),   \
("alqs-1",   0, "Artemis - Source is low quali with haloing"),   \
("ghq-5" , 900, "Gaia    - Source is high quali - add details"),  \
("gcg-5" ,   0, "Gaia    - Source is CGI"),   \
("dtd-3" ,   0, "Dione   - Source has fields")   \
]

outputPairs = [   \
(".mov", True , "mov_prores", "Prores 422 normal  (little lossy)" ),   \
(".mov", True , "mov_proreshq", "Prores 422 HQ  (Prores-normal size +25%)"),   \
(".mp4", True , "mp4", "MP4  H.264   (Lossy. Compresson quality option)"),   \
(".png", False, "png", "PNG            (Prores-normal size +100% to +300%)"),   \
(".jpg", False, "jpg", "Jpeg         (Lossy. Prores-normal size -90%)"),   \
(".tif", False, "8tif", "Tif 8bit   (Prores-normal size  +300% to +800%)") ,   \
(".tif", False, "16tif", "Tif 16bit")   \
]




#------------------------------- Create UI ---------------------------------------------------


myUI= rrGlobal.getGenericUI()
myUI.addItem(rrGlobal.genUIType.label,"infoLabel","")
myUI.setText("infoLabel","\nPlease select a video file or a folder with video files.\nThen choose scaling options.\n\n")

myUI.addItem(rrGlobal.genUIType.label,"labelF","")
myUI.setText("labelF","Choose source file:")
myUI.addItem(rrGlobal.genUIType.browseFile,"sourceF","")

myUI.addItem(rrGlobal.genUIType.label,"labelD","")
myUI.setText("labelD","OR source folder:")
myUI.addItem(rrGlobal.genUIType.browseDir,"sourceD","")

myUI.addItem(rrGlobal.genUIType.spacerV,"spacerC","")
myUI.addItem(rrGlobal.genUIType.label,"labelDivideC","")
myUI.setText("labelDivideC","_________________________________________________________")


myUI.addItem(rrGlobal.genUIType.label,"labelMode","")
myUI.setText("labelMode","AI Method:")
myUI.addItem(rrGlobal.genUIType.dropdown,"dropMode","")
for a, b, c in modePairs:
    myUI.addText("dropMode",c);


myUI.addItem(rrGlobal.genUIType.label,"labelOutput","")
myUI.setText("labelOutput","Output format:")
myUI.addItem(rrGlobal.genUIType.dropdown,"dropOutput","")
for a, b, c, d  in outputPairs:
    myUI.addText("dropOutput",d);

myUI.addItem(rrGlobal.genUIType.label,"labelmpg","")
myUI.setText("labelmpg","MP4 compression rate:")
myUI.addItem(rrGlobal.genUIType.spin,"mpgRate","")
myUI.setValueMin("mpgRate",0)
myUI.setValueMax("mpgRate",35)
myUI.setValue("mpgRate",15)

myUI.addItem(rrGlobal.genUIType.label,"labelGrain","")
myUI.setText("labelGrain","Grain ammount in 1/10%:")
myUI.addItem(rrGlobal.genUIType.spin,"grainRate","")
myUI.setValueMin("grainRate",0)
myUI.setValueMax("grainRate",100)
myUI.setValue("grainRate",0)

myUI.addItem(rrGlobal.genUIType.checkBox,"checkGPU","")
myUI.setValue("checkGPU",1)
myUI.setText("checkGPU","Use GPU")

myUI.addItem(rrGlobal.genUIType.spacerV,"spacerD","")
myUI.addItem(rrGlobal.genUIType.label,"labelDivideD","")
myUI.setText("labelDivideD","_________________________________________________________")

myUI.addItem(rrGlobal.genUIType.label,"labelResX","")
myUI.setText("labelResX","New video width:")
myUI.addItem(rrGlobal.genUIType.spin,"imgWidth","")
myUI.setValueMin("imgWidth",640)
myUI.setValueMax("imgWidth",9000)
myUI.setValue("imgWidth",1920)


myUI.addItem(rrGlobal.genUIType.label,"labelResY","")
myUI.setText("labelResY","Max height (only used if width setting would result in a too large video):")
myUI.addItem(rrGlobal.genUIType.spin,"maxImgHeight","")
myUI.setValueMin("maxImgHeight", 800)
myUI.setValueMax("maxImgHeight",4000)
myUI.setValue("maxImgHeight",2000)


myUI.addItem(rrGlobal.genUIType.checkBox,"checkResMult","")
myUI.setValue("checkResMult",1)
myUI.setText("checkResMult","Width and height have to be a multiple of 8 (tiny frame crop)")

myUI.addItem(rrGlobal.genUIType.checkBox,"checkSplitVideo","")
myUI.setValue("checkSplitVideo",1)
myUI.setText("checkSplitVideo","Video output: Split large sequences into multiple jobs")



myUI.addItem(rrGlobal.genUIType.spacerV,"spacerE","")
myUI.addItem(rrGlobal.genUIType.label,"labelDivideE","")
myUI.setText("labelDivideE","_________________________________________________________")


myUI.addItem(rrGlobal.genUIType.layoutH,"btnLayout","")
myUI.addItem(rrGlobal.genUIType.closeButton,"Create Job(s)","btnLayout")
myUI.addItem(rrGlobal.genUIType.closeButton,"Abort","btnLayout")


#------------------------------- Execute UI ---------------------------------------------------

myUI.execute()
if (myUI.value("Abort")==1):
    del myUI
    raise rrCleanExit()
    
isDirMode=False
sourceFile=myUI.text("sourceF");
if (len(sourceFile)==0):
    sourceFile=myUI.text("sourceD");
    isDirMode=True
    
AIMode= myUI.value("dropMode")
outFormat= myUI.value("dropOutput")
mpegRate= myUI.value("mpgRate")
grainRate= myUI.value("grainRate")
useGPU= myUI.value("checkGPU")
imgWidth= myUI.value("imgWidth")
maxImgHeight= myUI.value("maxImgHeight")
checkResMult= myUI.value("checkResMult")
checkSplitVideo= myUI.value("checkSplitVideo")

del myUI




if (len(sourceFile)==0):
    rrGlobal.writeLog(rrGlobal.logLvL.warning, "No source video/directory set!\nAborting.","")
    raise rrCleanExit()




#------------------------------- create video file list ---------------------------------------------------
from pymediainfo import MediaInfo
import os
import datetime



class resData:
    def __init__(self):
        self.y = 0
        self.x = 0 
        self.filename=""
        self.valid= False
        self.frameCount=1
    def getResolution(self,fileName):
        self.y = 0
        self.x = 0 
        self.frameCount=1
        self.filename=fileName
        media_info = MediaInfo.parse(fileName)
        for track in media_info.tracks:
            if track.track_type == "Video":
                if (track.height and track.width and track.frame_count):
                   self.x = int(track.width)
                   self.y = int(track.height)
                   self.frameCount = int(track.frame_count)
                   self.valid= True
                   #print(track.to_data())
                   #print("X: " + str(self.x) +"   Y: " + str(self.y) + "    Frames: "+str(track.frame_count))
                   
    
videoList= []
rrGlobal.progress_SetProgressA(0)
rrGlobal.progress_SetProgressB(0)
rrGlobal.progress_CancelHide()
rrGlobal.refreshUI()
 
vidCount=0
   
if (os.path.isfile(sourceFile)):
    resInfo= resData()
    resInfo.getResolution(sourceFile)  
    if (resInfo.valid):
        videoList.append(resInfo)
    rrGlobal.progress_SetMaxA(1)
    rrGlobal.progress_SetMaxB(1)
elif (os.path.isdir(sourceFile)):
    for file in os.listdir(sourceFile):
        vidCount= vidCount+1
    rrGlobal.progress_SetMaxA(vidCount)
    rrGlobal.progress_SetMaxB(vidCount)
    rrGlobal.progress_SetProgressB(0)
    vidCount=0
    
    for file in os.listdir(sourceFile):
        vidCount= vidCount+1
        rrGlobal.progress_SetProgressLabelA(vidCount,file )
        rrGlobal.refreshUI()
        if file.endswith(".avi") or file.endswith(".mpg") or file.endswith(".mp4") or file.endswith(".mpeg") or file.endswith(".mkv") or file.endswith(".wmv"):
            joinedName= os.path.join(sourceFile, file)
            #print(joinedName)
            resInfo= resData()
            resInfo.getResolution(joinedName)     
            if (resInfo.valid):
                videoList.append(resInfo)



#------------------------------- Create job(s) ---------------------------------------------------
renderApp= rrJob._RenderAppBasic()
renderApp.clear()
renderApp.name="VideoEnhanceAI"
renderApp.rendererName=""

vidCount=0

for video in videoList:
    vidCount= vidCount+1
    rrGlobal.progress_SetProgressB(vidCount)
    rrGlobal.refreshUI()
    appendFileName= "_"
    appendFileName= appendFileName+ "_" + str(imgWidth)

    newImageWidth= imgWidth
    if (checkResMult):
        newImageWidth= int( newImageWidth / 8 )
        newImageWidth= newImageWidth * 8
    newImageHeight= int(video.y * newImageWidth / video.x)
    if (checkResMult):
        newImageHeight= int( newImageHeight / 8 )
        newImageHeight= newImageHeight * 8
        
    if (newImageHeight> maxImgHeight):
        appendFileName= appendFileName+ "-" + str(maxImgHeight)
        newImageHeight= maxImgHeight
        if (checkResMult):
            newImageHeight= int( newImageHeight / 8 )
            newImageHeight= newImageHeight * 8
        newImageWidth= int(video.x * newImageHeight / video.y)
        if (checkResMult):
            newImageWidth= int( newImageWidth / 8 )
            newImageWidth= newImageWidth * 8
    
    useAiModeIdx=AIMode
    if (useAiModeIdx==0):
        matchFoundRes=0
        for i in range(len(modePairs)): 
            if (modePairs[i][1]<video.y and modePairs[i][1]>matchFoundRes):
                matchFoundRes=modePairs[i][1]
                useAiModeIdx=i
                
    if (useAiModeIdx==0):
        continue

    tmpString= modePairs[useAiModeIdx][0]
    if (tmpString.find('-')>0):
        tmpString= tmpString[:tmpString.find('-')]
    appendFileName= appendFileName+ "_" + tmpString
    if (outputPairs[outFormat][0]==".mp4"):
        appendFileName= appendFileName+ "_mpg" + str(mpegRate)
    if (grainRate>0):
        appendFileName= appendFileName+ "_gr" + str(grainRate)
    if (not useGPU):
        appendFileName= appendFileName+ "_cpu"
    
    source= video.filename
    sourceFile= source[:source.rfind('.')]
    if (sourceFile.endswith(".")):
        sourceFile= sourceFile[:len(sourceFile)-1]
    if (source.rfind('/')>0):
        sourceDir= source[:source.rfind('/')+1]
    else:
        sourceDir= source[:source.rfind('\\')+1]
    sourceFile= sourceFile[len(sourceDir):]
    if (outputPairs[outFormat][1]):     #is video file?
        targetName= sourceDir +  sourceFile + appendFileName + outputPairs[outFormat][0]
    else:
        targetName= sourceDir +  sourceFile + appendFileName + "/####" + outputPairs[outFormat][0]

        
    newJob=  rr.getNewJob()
    newJob.renderApp= renderApp
    newJob.sceneName= source
    newJob.imageFramePadding= 6
    newJob.imageFileName= targetName
    newJob.imageWidth= newImageWidth
    newJob.imageHeight= newImageHeight
    newJob.imageSingleOutputFile=outputPairs[outFormat][1]
    newJob.splitImageFileInto_DirFileExt(False)
    newJob.uiIsChecked= True
    newJob.seqStart= 0
    newJob.seqEnd= video.frameCount-1;
    
    appendFileName=appendFileName[1:]
    appendFileName=appendFileName[:len(appendFileName)-1]
    newJob.layer= appendFileName
    
    newJob.customDataAppend_Str("AIMethod", modePairs[useAiModeIdx][0])
    newJob.customDataAppend_Str("OutFormat", outputPairs[outFormat][2])
    if (outputPairs[outFormat][0]==".mp4"):
        newJob.customDataAppend_Str("CompressionRate", str(mpegRate))
    if (grainRate>0):
        newJob.customDataAppend_Str("GrainAmmount", str(grainRate))
    if (useGPU):
        newJob.customDataAppend_Str("rrSubmitterParameter", " GPUrequired=0~1")
        
    newJob.customDataAppend_Str("rrSubmitterParameter", " RequiredMemory=1~6")
    newJob.customDataAppend_Str("rrSubmitterParameter", " MaxCrashes=1~2")
    newJob.customDataAppend_Str("rrSubmitterParameter", " SeqDivMINComp=1~5000")
    newJob.customDataAppend_Str("rrSubmitterParameter", " SeqDivMAXComp=1~5000")
    if (outputPairs[outFormat][1]):     #is video file?
        newJob.customDataAppend_Str("rrSubmitterParameter", " DoNotCheckForFrames=0~1")
    if (checkSplitVideo and outputPairs[outFormat][1]):
        splitSize= 20000
        newJob.seqStart= 0
        newJob.seqEnd = -1
        while (video.frameCount-1 - newJob.seqEnd > splitSize):
            newJob.seqEnd= newJob.seqEnd + splitSize
            rr.jobAll_set(999,newJob);  
            newJob.seqStart= newJob.seqStart + splitSize
        newJob.seqEnd= video.frameCount-1;
        rr.jobAll_set(999,newJob); 
    else:
        rr.jobAll_set(999,newJob);  # If the index of the job is higher than the number of jobs,
                                # then a new job is added to the end of the job list. So I use 999.
